==> 0.9.12.tar.gz: lib/model/default_editors.dart <==
part of fields.model;

class Bit16Editor extends FieldEditor<int> {
  
  var _expander;
  var _display;
  var _dropdown;
  var _element;
  get element => _element;
  
  Bit16Editor(Field field) : super(field) {
    _element = span(() {
      span(() {
        _expander = toggleButton(() {
          span()..innerHtml = '&#9655;'..classes = ['up'];
          span()..innerHtml = '&#9698;'..classes = ['down'];
        })
        ..classes = ['expander']
        ..onToggle.listen((_) => refreshElement());
        _display = input()
        ..type = 'text'
        ..spellcheck = false
        ..onInput.listen((_) {
          try {
            this.value = Transform.transform(_display.value, String, int);
            _display.classes.remove('error');
          } catch(_) {
            _display.classes.add('error');
          }
        });
      })..style.display = 'flex';      
      _dropdown = div()
      ..hidden = true
      ..style.display = 'flex'
      ..style.flexWrap = 'wrap';
    })
    ..style.display = 'flex'
    ..style.flexDirection = 'column';
  }
  
  void refreshElement() {
    final String displayValue = '0x${value.toRadixString(16)}';
    if (_display.value != displayValue) {
      _display.value = displayValue;
    }
    _dropdown.hidden = !_expander.isDown;
    if (_expander.isDown) {
      _refreshBits();
    }
  }
  
  void _refreshBits() {
    _dropdown.children.clear();
    for (int i = 15; i >= 0; --i) {
      final int bits = 0x0000 | (1 << i);
      final bitField = new Field<bool>('bit$i', () => (value & bits) != 0,
          setter: (bool bit) {
            if (bit) {
              value |= bits;
            } else {
              value &= ~bits;
            }
          });
      _dropdown.append(new BoolEditor(bitField).element); 
    }
  }
}

class BoolEditor extends FieldEditor<bool> {
  
  static final _checkBox = new String.fromCharCode(9744);
  static final _check = new String.fromCharCode(9745);
  
  var _toggle;
  var _element;
  get element => _element;
  
  BoolEditor(Field field) : super(field) {
    final desc = (field.description == null) ? '' : field.description;
    _element = span(() {
      _toggle = toggleButton(() {
        span()
        ..text = '$_checkBox $desc'
        ..classes = ['up'];
        span()
        ..text = '$_check $desc'
        ..classes = ['down'];
      })..onToggle.listen((bool isDown) {
        if (isEnabled) {
          this.value = isDown;
        }
      });
    });
  }
      
  void refreshElement() {
    _toggle
    ..disabled = !isEnabled
    ..isDown = value;
  }
}

class ColorEditor extends FieldEditor<String> {
  
  var _picker;
  var _display;
  var _element;
  get element => _element;
  
  ColorEditor(Field field) : super(field) {
    _element = span(() {
      _picker = input()
      ..type = 'color'
      ..onChange.listen((_) {
        _display.value = _picker.value;         
        this.value = _picker.value;
      });
      _display = input()
      ..type = 'text'
      ..spellcheck = false
      ..onInput.listen((_) {
        try {
          this.value = _display.value;
          _display.classes.remove('error');
        } catch(_) {
          _display.classes.add('error');
        }
      });
    })..style.display = 'flex';
  }
  
  void refreshElement() {
    _picker
    ..disabled = !isEnabled
    ..value = value;
    if (_display.value != value) {
      _display.value = value;
    }
  }
}

class DateTimeEditor extends FieldEditor<DateTime> {
  
  var _input;
  get element => _input;
  
  DateTimeEditor(Field field) : super(field) {
    _input = input()
    ..type = 'date' // TODO: investigate 'datetime-local'
    ..onChange.listen((_) {
      this.value = _input.valueAsDate;
    });
  }
  
  void refreshElement() {
    _input
    ..disabled = !isEnabled
    ..valueAsDate = value;
  }
}

class DefaultEditor extends FieldEditor<String> {
  
  var _input;
  get element => _input;
    
  DefaultEditor(Field field) : super(field) {
    _input = input()
    ..type = 'text'
    ..onInput.listen((_) {
      try {
        this.value = _input.value;
        _input.classes.remove('error');
      } catch(_) {
        _input.classes.add('error');
      }
    });
  }
  
  void refreshElement() {
    _input.disabled = !isEnabled;
    if (_input.value != value) {
      _input.value = value;
    }
  }
}

class NumberEditor extends FieldEditor<num> {
  
  var _input;
  get element => _input;
  Range _range;
  
  NumberEditor(Field field) : super(field) {
    _input = input()
    ..type = 'number'
    ..onChange.listen((_) {
      final num n = _input.valueAsNumber;
      if (!n.isFinite || (_range != null && !_range.check(n))) {
        _input.valueAsNumber = this.value;
        return;
      }
      this.value = n;
    });
    
    final match = (a) => a is Range;
    if (field.annotations.any(match)) {
      _range = field.annotations.firstWhere(match);
      _range.applyTo(_input);
    }
  }
  
  void refreshElement() {
    _input
    ..disabled = !isEnabled
    ..valueAsNumber = value;
  }
}

class RangeEditor extends FieldEditor<num> {
  
  var _rangeInput;
  var _numberInput;
  var _element;
  get element => _element;
  Range _range;
  
  RangeEditor(Field field) : super(field) {
    _element = span(() {
      _rangeInput = input()
      ..type = 'range'
      ..style.flexShrink = '0'
      ..style.width = '60%'
      ..onChange.listen((_) {
        this.value = _rangeInput.valueAsNumber;
      });      
      _numberInput = input()
      ..type = 'number'
      ..style.width = '40%'
      ..onChange.listen((_) {
        final num n = _numberInput.valueAsNumber;
        if (!n.isFinite || (_range != null && !_range.check(n))) {
          _numberInput.valueAsNumber = this.value;
          return;
        }
        this.value = n;
      });
    })..style.display = 'flex';

    final match = (a) => a is Range;
    if (field.annotations.any(match)) {
      _range = field.annotations.firstWhere(match);
      _range.applyTo(_rangeInput);
      _range.applyTo(_numberInput);
    }
  }
    
  void refreshElement() {
    _rangeInput
    ..disabled = !isEnabled
    ..valueAsNumber = value;
    _numberInput
    ..disabled = !isEnabled
    ..valueAsNumber = value;
  }
}

class SelectEditor extends FieldEditor<String> {
  
  Values _values;
  
  var _select;
  get element => _select;   
  
  SelectEditor(Field field) : super(field) {
    _select = select()
    ..onChange.listen((_) {
      this.value = _select.value;
    });
  }
  
  void refreshElement() {
    refreshOptions();
    _select
    ..disabled = !isEnabled || _select.children.isEmpty
    ..value = value;
  }
  
  void refreshOptions() {
    final isValues = (a) => a is Values;
    if (field.annotations.any(isValues)) {
      _values = field.annotations.firstWhere(isValues);        
    }
    if (_values != null) {
      _select.children.clear();
      for (var v in _values.values) {
        String valueString;
        if (field.transform != null) {
          valueString = field.transform(v);
        } else if (Transform.canTransform(v.runtimeType, String)) {
          valueString = Transform.transform(v, v.runtimeType, String);
        } else {
          valueString = v.toString();
        }
        final option = dom.createElement('option')
        ..text = valueString
        ..value = valueString;
        _select.append(option);
      }
    }
  }
}

==> 0.11.0.tar.gz: lib/model/default_editors.dart <==
part of fields.model;

class BitEditor extends FieldEditor<int> {
  
  Range _range;
  var _expander;
  var _display;
  var _dropdown;
  var _element;
  get element => _element;
  
  BitEditor(Field field) : super(field) {
    _element = span(() {
      span(() {
        _expander = toggleButton(() {
          span()..innerHtml = '&#9655;'..classes = ['up'];
          span()..innerHtml = '&#9698;'..classes = ['down'];
        })
        ..classes = ['expander']
        ..onToggle.listen((_) => refreshElement());
        _display = input()
        ..type = 'text'
        ..spellcheck = false
        ..onInput.listen((_) {
          try {
            this.value = Transform.transform(_display.value, String, int);
            _display.classes.remove('error');
          } catch(_) {
            _display.classes.add('error');
          }
        });
      })..style.display = 'flex';      
      _dropdown = div()
      ..hidden = true
      ..style.display = 'flex'
      ..style.flexWrap = 'wrap';
    })
    ..style.display = 'flex'
    ..style.flexDirection = 'column';    
    final isRange = (a) => a is Range;
    if (field.annotations.any(isRange)) {
      _range = field.annotations.firstWhere(isRange);
    }
  }
  
  void refreshElement() {
    final String displayValue = '0x${value.toRadixString(16)}';
    if (_display.value != displayValue) {
      _display.value = displayValue;
    }
    _dropdown.hidden = !_expander.isDown;
    if (_expander.isDown) {
      _refreshBits();
    }
  }
  
  void _refreshBits() {
    _dropdown.children.clear();
    int i = 15;
    if (_range != null && _range.max != null) {
      i = _range.max;      
    }
    for (i; i >= 0; --i) {
      final int bits = (1 << i);
      final bitField = new Field<bool>('bit$i', () => (value & bits) != 0,
          setter: (bool bit) {
            if (bit) {
              value |= bits;
            } else {
              value &= ~bits;
            }
          });
      _dropdown.append(new BoolEditor(bitField).element); 
    }
  }
}

class BoolEditor extends FieldEditor<bool> {
  
  static final _checkBox = new String.fromCharCode(9744);
  static final _check = new String.fromCharCode(9745);
  
  var _toggle;
  var _element;
  get element => _element;
  
  BoolEditor(Field field) : super(field) {
    final desc = (field.description == null) ? '' : field.description;
    _element = span(() {
      _toggle = toggleButton(() {
        span()
        ..text = '$_checkBox $desc'
        ..classes = ['up'];
        span()
        ..text = '$_check $desc'
        ..classes = ['down'];
      })..onToggle.listen((bool isDown) {
        if (isEnabled) {
          this.value = isDown;
        }
      });
    });
  }
      
  void refreshElement() {
    _toggle
    ..disabled = !isEnabled
    ..isDown = value;
  }
}

class ColorEditor extends FieldEditor<String> {
  
  var _picker;
  var _display;
  var _element;
  get element => _element;
  
  ColorEditor(Field field) : super(field) {
    _element = span(() {
      _picker = input()
      ..type = 'color'
      ..onChange.listen((_) {
        _display.value = _picker.value;         
        this.value = _picker.value;
      });
      _display = input()
      ..type = 'text'
      ..spellcheck = false
      ..onInput.listen((_) {
        try {
          this.value = _display.value;
          _display.classes.remove('error');
        } catch(_) {
          _display.classes.add('error');
        }
      });
    })..style.display = 'flex';
  }
  
  void refreshElement() {
    _picker
    ..disabled = !isEnabled
    ..value = value;
    if (_display.value != value) {
      _display.value = value;
    }
  }
}

class DateTimeEditor extends FieldEditor<DateTime> {
  
  var _input;
  get element => _input;
  
  DateTimeEditor(Field field) : super(field) {
    _input = input()
    ..type = 'date' // TODO: investigate 'datetime-local'
    ..onChange.listen((_) {
      this.value = _input.valueAsDate;
    });
  }
  
  void refreshElement() {
    _input
    ..disabled = !isEnabled
    ..valueAsDate = value;
  }
}

class DefaultEditor extends FieldEditor<String> {
  
  var _input;
  get element => _input;
    
  DefaultEditor(Field field) : super(field) {
    _input = input()
    ..type = 'text'
    ..onInput.listen((_) {
      try {
        this.value = _input.value;
        _input.classes.remove('error');
      } catch(_) {
        _input.classes.add('error');
      }
    });
  }
  
  void refreshElement() {
    _input.disabled = !isEnabled;
    if (_input.value != value) {
      _input.value = value;
    }
  }
}

class NumberEditor extends FieldEditor<num> {
  
  var _input;
  get element => _input;
  Range _range;
  
  NumberEditor(Field field) : super(field) {
    _input = input()
    ..type = 'number'
    ..onChange.listen((_) {
      final num n = _input.valueAsNumber;
      if (!n.isFinite || (_range != null && !_range.check(n))) {
        _input.valueAsNumber = this.value;
        return;
      }
      this.value = n;
    });
    
    final match = (a) => a is Range;
    if (field.annotations.any(match)) {
      _range = field.annotations.firstWhere(match);
      _range.applyTo(_input);
    }
  }
  
  void refreshElement() {
    _input
    ..disabled = !isEnabled
    ..valueAsNumber = value;
  }
}

class RangeEditor extends FieldEditor<num> {
  
  var _rangeInput;
  var _numberInput;
  var _element;
  get element => _element;
  Range _range;
  
  RangeEditor(Field field) : super(field) {
    _element = span(() {
      _rangeInput = input()
      ..type = 'range'
      ..style.flexShrink = '0'
      ..style.width = '60%'
      ..onChange.listen((_) {
        this.value = _rangeInput.valueAsNumber;
      });      
      _numberInput = input()
      ..type = 'number'
      ..style.width = '40%'
      ..onChange.listen((_) {
        final num n = _numberInput.valueAsNumber;
        if (!n.isFinite || (_range != null && !_range.check(n))) {
          _numberInput.valueAsNumber = this.value;
          return;
        }
        this.value = n;
      });
    })..style.display = 'flex';

    final match = (a) => a is Range;
    if (field.annotations.any(match)) {
      _range = field.annotations.firstWhere(match);
      _range.applyTo(_rangeInput);
      _range.applyTo(_numberInput);
    }
  }
    
  void refreshElement() {
    _rangeInput
    ..disabled = !isEnabled
    ..valueAsNumber = value;
    _numberInput
    ..disabled = !isEnabled
    ..valueAsNumber = value;
  }
}

class SelectEditor extends FieldEditor<String> {
  
  Values _values;
  
  var _select;
  get element => _select;   
  
  SelectEditor(Field field) : super(field) {
    _select = select()
    ..onChange.listen((_) {
      this.value = _select.value;
    });
  }
  
  void refreshElement() {
    refreshOptions();
    _select
    ..disabled = !isEnabled || _select.children.isEmpty
    ..value = value;
  }
  
  void refreshOptions() {
    final isValues = (a) => a is Values;
    if (field.annotations.any(isValues)) {
      _values = field.annotations.firstWhere(isValues);        
    }
    if (_values != null) {
      _select.children.clear();
      for (var v in _values.values) {
        String valueString;
        if (field.transform != null) {
          valueString = field.transform(v);
        } else if (Transform.canTransform(v.runtimeType, String)) {
          valueString = Transform.transform(v, v.runtimeType, String);
        } else {
          valueString = v.toString();
        }
        final option = dom.createElement('option')
        ..text = valueString
        ..value = valueString;
        _select.append(option);
      }
    }
  }
}
